/***************************************************************************
 *   Copyright (C) 2006-2007 by Ricardo I. Vieitez - Fernando Beradi       *
 *   ricardo@mundolinux.org - fernando@zerial.org                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation (version 2 of the License)               *
 *                                                                         *
 *   Note that the only valid version of the GPL as far as this server is  *
 *   the version 2 of the license but not any other (i.e. v2, not 1 or 2.2 *
 *   or 3 or whatever), unless explicitly otherwise stated.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <iostream>
#include <stdlib.h>

#include <sys/socket.h>
#include <sys/wait.h>
#include <arpa/inet.h>

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <time.h>
#include <errno.h>

#include "server.h"

#define _LOG 10

#define MIN_PORT 30
#define MAX_PORT 15000

#define MIN_SLEN_ROOT 6
#define MAX_SLEN_ROOT 10000

c_WebServer::c_WebServer() {
	/* Initialize values */
	m_iPort=0;
	m_iSockfd=0;
	m_iSinSize=0;
	m_psRoot=NULL;
	m_pServAddr=0;
	m_pClientAddr=0;
}

c_WebServer::c_WebServer(int iPort) {
	c_WebServer(); /* Initialize values */
	setPort(iPort); /* Set port */
}

paramErr c_WebServer::setPort(int iPort) {
	/* Verify port range */
	if (iPort<MIN_PORT
		|| iPort>MAX_PORT) return BAD_PORT;
	/* Set port */
	m_iPort=iPort;
	/* Verify if it was set */
	if (whatPort()!=iPort) return UNSET_PORT;
	return OK;
}

paramErr c_WebServer::setRoot(char* psRoot) {
	/* Veify for a right address */
	if (!psRoot) return BAD_ROOT;

	/* Set 0 at the end of the string */
	*((psRoot)+(sizeof(psRoot)-1))='\0';

	/* Verify strlens */
	if (strlen(psRoot)<MIN_SLEN_ROOT 
		|| strlen(psRoot)>MIN_SLEN_ROOT)
		{ return BAD_ROOT; }

	/* Create a new string */
	m_psRoot=new char[strlen(psRoot)];

	/* Verify the memory was allocated */
	if (m_psRoot) {
		/* Copy the strings */
		strncpy(m_psRoot,psRoot,sizeof(m_psRoot));
	} else return UNSET_ROOT;

	/* Verify if it was set */
	if (strcmp(whatRoot(),psRoot)!=0) return UNSET_ROOT;
	return OK;
}

paramErr c_WebServer::operator= (struct c_ws_config& config) {
	paramErr lastCall;
	if ((lastCall=this->setPort(config.port))!=OK) {
		return lastCall;
	}
	if ((lastCall=this->setRoot(config.root))!=OK) {
		return lastCall;
	}
	
	return OK;
}

int c_WebServer::whatPort(void) {
	return m_iPort;
}

char* c_WebServer::whatRoot(void) {
	return m_psRoot;
}

servErr c_WebServer::listen() {
	/* Open socket */
	if ((m_iSockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		return SOCKET;
	}

	m_pServAddr=new struct sockaddr_in;

	m_pServAddr->sin_family = AF_INET;
	m_pServAddr->sin_port = htons(m_iPort);
	m_pServAddr->sin_addr.s_addr = INADDR_ANY;
	bzero(&(m_pServAddr->sin_zero), 8);
	
	/* Bind address */
	if (bind(m_iSockfd, (struct sockaddr *) m_pServAddr, 
		sizeof(struct sockaddr)) == -1)
		return BIND;
	
	/* Listen */	
	if (::listen(m_iSockfd, _LOG) == -1)
	{
		return LISTEN;
	}
	
	/* Infinite while */
	while(1)
	{
		m_iSinSize = sizeof(struct sockaddr_in);
		
		m_pClientAddr=new struct sockaddr_in;
		
		/* Wait for connections */
		if ((m_iNewfd = accept(m_iSockfd, (struct sockaddr *)m_pClientAddr,(socklen_t*) &m_iSinSize)) == -1)
		{
			std::cerr << "Cannot accept requests. Error.";
			continue;
		}
		printf("HTTPD: connection from: %s\n", inet_ntoa(m_pClientAddr->sin_addr));
		if (!fork())
		{ /* Here starts the subprocess. It sends messages thought newfd */
			~(*this);
		}
		
		close(m_iNewfd); /* Closes connection */
		/* Wait for PID */
		while(waitpid(-1,NULL,WNOHANG) > 0);
		
	}
}

servErr c_WebServer::operator++ (void) {
	return listen();
}
